<?php

App::uses('BuzzBookingsAppModel', 'BuzzBookings.Model');

class ActivityPackage extends BuzzBookingsAppModel {

/**
 * Behaviors
 *
 * @var array
 */
	public $actsAs = array(
		'BuzzTranslate.Translatable'
	);

/**
 * Belongs to associations
 *
 * @var array
 */
	public $belongsTo = array(
		'Activity' => array(
			'className' => 'BuzzBookings.Activity'
		)
	);

/**
 * HABTM associations
 *
 * @var array
 */
	public $hasAndBelongsToMany = array(
		'UpsellApplicableActivity' => array(
			'className' => 'BuzzBookings.Activity',
			'joinTable' => 'activity_packages_upsell_applicable_activities'
		),
		'UpsellExclusion' => array(
			'className' => 'BuzzBookings.ActivityPackage',
			'joinTable' => 'activity_packages_upsell_exclusions'
		)
	);

/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'api_reference' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'multi_experience' => array(
				'rule' => 'validateApiReferences',
				'message' => 'Invalid reference codes'
			)
		),
		'api_discount_code' => array(
			'max' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long',
				'allowEmpty' => true
			)
		),
		'name' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'max' => array(
				'rule' => array('maxLength', 150),
				'message' => 'No more than 150 characters long'
			)
		),
		'activity_id' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		),
		'peak_price' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'decimal' => array(
				'rule' => '|^\d+(\.\d{2})?$|',
				'message' => 'Decimal numbers only'
			)
		),
		'off_peak_price' => array(
			'decimal' => array(
				'rule' => '|^\d+(\.\d{2})?$|',
				'message' => 'Decimal numbers only',
				'allowEmpty' => true
			)
		),
		'upsell_price' => array(
			'decimal' => array(
				'rule' => '|^\d+(\.\d{2})?$|',
				'message' => 'Decimal numbers only',
				'allowEmpty' => true
			)
		),
		'sequence' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'max' => array(
				'rule' => array('naturalNumber', true)
			)
		)
	);

/**
 * Image slots
 *
 * @var array
 */
	public $imageSlots = [
		'main' => [
			'slots' => 1,
			'fields' => []
		]
	];

/**
 * Read for edit
 *
 * @param int $id
 * @param array $query
 * @return array
 */
	public function readForEdit($id, $query = []) {
		$query['contain'][] = 'UpsellApplicableActivity';
		return parent::readForEdit($id, $query);
	}

/**
 * Custom validation method for checking the supplied API references.
 *
 * Only multi-experience activities can have a list of reference numbers.
 *
 * @param array $data
 * @return bool
 */
	public function validateApiReferences(array $data) {
		$check = array_pop($data);

		if (!empty($this->data[$this->alias]['activity_id'])) {

			if ((int)$this->data[$this->alias]['activity_id'] !== $this->Activity->getMultiExperience()) {
				return preg_match('|^\d+$|', $check) === 1;
			} else {
				return preg_match('|^(\d+\[\d+(\.\d{2})?\],?\ ?)+$|', $check) === 1;
			}

		}

		return;
	}

/**
 * After find logic
 *
 * @param array $results
 * @param bool $primary
 * @return array
 */
	public function afterFind($results, $primary = false) {
		foreach ($results as &$result) {
			// For packages with a single price create a price attribute. This
			// is to simplify things elsewhere.
			if (!empty($result[$this->alias]['peak_price'])) {
				$result[$this->alias]['price'] = null;
				if (
					$result[$this->alias]['peak_price'] === $result[$this->alias]['off_peak_price']
					|| empty($result[$this->alias]['off_peak_price'])
					|| $this->Activity->getMultiExperience() === (int)$result[$this->alias]['activity_id']
				) {
					$result[$this->alias]['price'] = $result[$this->alias]['peak_price'];
				}
			}
		}
		return $results;
	}

}
